<?php
if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

// Admin Settings Menu

add_action( 'admin_menu', 'bbp_voting_settings_menu' );
function bbp_voting_settings_menu(){
  add_options_page( 'bbPress Voting',
                'bbPress Voting', 
                'manage_options', 
                'bbp_voting', 
                'bbp_voting_settings_page' );
}

// Register Settings

add_action( 'admin_init', 'update_bbp_voting_settings' );
function update_bbp_voting_settings() {
    global $bbp_voting_hooks;
    foreach($bbp_voting_hooks as $bbp_voting_hook => $bbp_voting_hook_type) {
        register_setting( 'bbp_voting_settings', $bbp_voting_hook );
    }
}

// Admin Settings Page

function bbp_voting_settings_page(){
?>
<div class="wrap">
  <h1>bbPress Voting</h1>
  <form method="post" action="options.php">
    <?php settings_fields( 'bbp_voting_settings' ); ?>
    <?php do_settings_sections( 'bbp_voting_settings' ); ?>
    <h2>Look & Feel</h2>
    <table class="form-table">

        <tr valign="top">
            <th scope="row">Show Labels:</th>
            <td>
                <input type="checkbox" name="bbp_voting_show_labels" value="true" <?php if(get_option('bbp_voting_show_labels')=='true') echo 'checked'; ?> />
                <p class="description">Show the labels that describe what up and down mean?</p>
            </td>
        </tr>
        
        <tr valign="top">
            <th scope="row">Upvote Label:</th>
            <td>
                <input type="text" name="bbp_voting_helpful" value="<?php echo get_option( 'bbp_voting_helpful' ); ?>" data-lpignore="true" />
                <p class="description">Change the upvote label from "Helpful" to something else.</p>
            </td>
        </tr>
        
        <tr valign="top">
            <th scope="row">Downvote Label:</th>
            <td>
                <input type="text" name="bbp_voting_not_helpful" value="<?php echo get_option( 'bbp_voting_not_helpful' ); ?>" data-lpignore="true" />
                <p class="description">Change the downvote label from "Not Helpful" to something else.</p>
            </td>
        </tr>

    </table>
    <hr />
    <h2>Behavior</h2>
    <table class="form-table">

        <tr valign="top">
            <th scope="row">Disable Voting on Replies:</th>
            <td>
                <input type="checkbox" name="bbp_voting_only_topics" value="true" <?php if(get_option('bbp_voting_only_topics')=='true') echo 'checked'; ?> />
                <p class="description">You can override this at the forum level.</p>
            </td>
        </tr>

        <tr valign="top">
            <th scope="row">Disable Voting on Topics:</th>
            <td>
                <input type="checkbox" name="bbp_voting_only_replies" value="true" <?php if(get_option('bbp_voting_only_replies')=='true') echo 'checked'; ?> />
                <p class="description">You can override this at the forum level.</p>
            </td>
        </tr>

        <tr valign="top">
            <th scope="row">View-Only Scores on Closed Topics:</th>
            <td>
                <input type="checkbox" name="bbp_voting_disable_voting_on_closed_topic" value="true" <?php if(get_option('bbp_voting_disable_voting_on_closed_topic')=='true') echo 'checked'; ?> />
                <p class="description">After a topic is closed, scores will display (if configured to), but new votes for the topic or the topic's replies will be rejected.</p>
            </td>
        </tr>
        
        <tr valign="top">
            <th scope="row">Sort Topics by Votes:</th>
            <td>
                <input type="checkbox" name="sort_bbpress_topics_by_votes" value="true" <?php if(get_option('sort_bbpress_topics_by_votes')=='true') echo 'checked'; ?> />
                <p class="description">Sort topics in a forum using their voting scores? (highest voted topics on top)</p>
            </td>
        </tr>

        <tr valign="top">
            <th scope="row">Sort Replies by Votes:</th>
            <td>
                <input type="checkbox" name="sort_bbpress_replies_by_votes" value="true" <?php if(get_option('sort_bbpress_replies_by_votes')=='true') echo 'checked'; ?> />
                <p class="description">Sort replies on a topic using their voting scores? (highest voted replies on top)</p>
            </td>
        </tr>

        <tr valign="top">
            <th scope="row">Admin Bypass:</th>
            <td>
                <input type="checkbox" name="bbp_voting_admin_bypass" value="true" <?php if(get_option('bbp_voting_admin_bypass')=='true') echo 'checked'; ?> />
                <p class="description">Allow any administrator user to vote as much as they want?</p>
            </td>
        </tr>

    </table>
    <?php submit_button(); ?>
  </form>
</div>
<?php
}

add_action('add_meta_boxes', 'bbp_voting_metaboxes');
function bbp_voting_metaboxes() {
    add_meta_box(
        'bbp_voting',
        'bbPress Voting',
        'bbp_voting_forum_metabox',
        'forum',
        'side',
        'low'
    );
}

function bbp_voting_forum_metabox() {
    $post_id = get_the_ID();
    $options = array(
        '' => 'Default',
        'true' => 'Enable',
        'false' => 'Disable'
    );
    ?>
    <p class="description">
        Enable or disable voting on topics or replies, only for this forum.
    </p>
    <p>
		<strong class="label">Voting on Topics:</strong>
		<select type="checkbox" name="bbp_voting_forum_enable_topics" value="true" id="bbp_voting_forum_enable_topics" class="bbp_dropdown">
            <?php
            $selected = get_post_meta( $post_id, 'bbp_voting_forum_enable_topics', true);
            foreach($options as $value => $label) {
                echo '<option value="'.$value.'" ';
                if($selected==$value) echo 'selected';
                echo '>'.$label.'</option>';
            }
            ?>
        </select>
    </p>
    <p>
		<strong class="label">Voting on Replies:</strong>
		<select type="checkbox" name="bbp_voting_forum_enable_replies" value="true" id="bbp_voting_forum_enable_replies" class="bbp_dropdown">
            <?php
            $selected = get_post_meta( $post_id, 'bbp_voting_forum_enable_replies', true);
            foreach($options as $value => $label) {
                echo '<option value="'.$value.'" ';
                if($selected==$value) echo 'selected';
                echo '>'.$label.'</option>';
            }
            ?>
		</select>
    </p>
    <p class="description">
        Enable or disable sorting based on votes on topics or replies, only for this forum.
    </p>
    <p>
		<strong class="label">Sort Topics by Votes:</strong>
		<select type="checkbox" name="sort_bbpress_topics_by_votes_on_forum" value="true" id="sort_bbpress_topics_by_votes_on_forum" class="bbp_dropdown">
            <?php
            $selected = get_post_meta( $post_id, 'sort_bbpress_topics_by_votes_on_forum', true);
            foreach($options as $value => $label) {
                echo '<option value="'.$value.'" ';
                if($selected==$value) echo 'selected';
                echo '>'.$label.'</option>';
            }
            ?>
        </select>
    </p>
    <p>
		<strong class="label">Sort Replies by Votes:</strong>
		<select type="checkbox" name="sort_bbpress_replies_by_votes_on_forum" value="true" id="sort_bbpress_replies_by_votes_on_forum" class="bbp_dropdown">
            <?php
            $selected = get_post_meta( $post_id, 'sort_bbpress_replies_by_votes_on_forum', true);
            foreach($options as $value => $label) {
                echo '<option value="'.$value.'" ';
                if($selected==$value) echo 'selected';
                echo '>'.$label.'</option>';
            }
            ?>
		</select>
    </p>
    <?php 
}

add_action( 'save_post', 'bbp_voting_save_forum_metabox' );
function bbp_voting_save_forum_metabox($post_id) {
    if (array_key_exists('bbp_voting_forum_enable_topics', $_POST)) {
        update_post_meta(
            $post_id,
            'bbp_voting_forum_enable_topics',
            $_POST['bbp_voting_forum_enable_topics']
        );
    }
    if (array_key_exists('bbp_voting_forum_enable_replies', $_POST)) {
        update_post_meta(
            $post_id,
            'bbp_voting_forum_enable_replies',
            $_POST['bbp_voting_forum_enable_replies']
        );
    }
    if (array_key_exists('sort_bbpress_topics_by_votes_on_forum', $_POST)) {
        update_post_meta(
            $post_id,
            'sort_bbpress_topics_by_votes_on_forum',
            $_POST['sort_bbpress_topics_by_votes_on_forum']
        );
    }
    if (array_key_exists('sort_bbpress_replies_by_votes_on_forum', $_POST)) {
        update_post_meta(
            $post_id,
            'sort_bbpress_replies_by_votes_on_forum',
            $_POST['sort_bbpress_replies_by_votes_on_forum']
        );
    }
}