<?php
if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

// Enqueue Scripts

add_action('wp_enqueue_scripts', 'bbp_voting_styles');

function bbp_voting_styles() {
    wp_enqueue_style( 'bbp-voting-css', trailingslashit(plugin_dir_url(__FILE__)) . 'css/bbp-voting.css', array() );
}

add_action('wp_enqueue_scripts', 'bbp_voting_scripts');

function bbp_voting_scripts() {
    wp_enqueue_style( 'bbp-voting-css', trailingslashit(plugin_dir_url(__FILE__)) . 'css/bbp-voting.css', array() );
    if(function_exists( 'is_amp_endpoint' ) && is_amp_endpoint()) {
        // AMP = No JS
    } else {
        wp_enqueue_script( 'bbp-voting-js', trailingslashit(plugin_dir_url(__FILE__)) . 'js/bbp-voting.js', array('jquery') );
        wp_localize_script( 'bbp-voting-js', 'ajax_object', array( 'ajax_url' => admin_url( 'admin-ajax.php' ) ) );
    }
}

// Vote Buttons and Score

add_action('bbp_theme_before_topic_title', 'bbp_voting_buttons');
add_action('bbp_template_before_lead_topic', 'bbp_voting_buttons');
add_action('bbp_theme_before_topic_author_details', 'bbp_voting_buttons');
add_action('bbp_theme_before_reply_author_details', 'bbp_voting_buttons');

function bbp_voting_buttons() {
    $this_post_type = bbp_voting_get_current_post_type();
    $topic_post_type = bbp_get_topic_post_type();
    $reply_post_type = bbp_get_reply_post_type();
    switch($this_post_type) {
        case $topic_post_type:
            $post = bbpress()->topic_query->post;
        break;
        case $reply_post_type:
            $post = bbpress()->reply_query->post;
        break;
    }
    if(!empty($post)) {
        $post_id = $post->ID;
        switch($this_post_type) {
            case $topic_post_type:
                $forum_id = bbp_get_topic_forum_id($post_id);
                $post_setting = get_post_meta( $forum_id, 'bbp_voting_forum_enable_topics', true);
                $broad_disable = apply_filters('bbp_voting_only_replies', false);
            break;
            case $reply_post_type:
                $forum_id = bbp_get_reply_forum_id($post_id);
                $post_setting = get_post_meta( $forum_id, 'bbp_voting_forum_enable_replies', true);
                $broad_disable = apply_filters('bbp_voting_only_topics', false);
            break;
        }
        if(!empty($post_setting)) {
            // Forum-specific override is set (not Default)
            if($post_setting === 'false') return;
        } else {
            // Use broad disable settings
            if($broad_disable === true) return;
        }
        // Filter Hook: 'bbp_voting_allowed_on_forum'
        if(!apply_filters('bbp_voting_allowed_on_forum', true, $forum_id)) return;
        // Done with checks... let's do this
        $score = (int) get_post_meta($post_id, 'bbp_voting_score', true);
        $ups = (int) get_post_meta($post_id, 'bbp_voting_ups', true);
        $downs = (int) get_post_meta($post_id, 'bbp_voting_downs', true);
        // Check for and correct discrepancies
        $calc_score = $ups + $downs;
        if($score > $calc_score) {
            $diff = $score - $calc_score;
            $ups += $diff;
            update_post_meta($post_id, 'bbp_voting_ups', $ups);
        }
        // Get user's vote by ID or IP
        $voting_log = get_post_meta($post_id, 'bbp_voting_log', true);
        $voting_log = is_array($voting_log) ? $voting_log : array(); // Set up new array
        $client_ip = $_SERVER['REMOTE_ADDR'];
        $identifier = is_user_logged_in() ? get_current_user_id() : $client_ip;
        $existing_vote = array_key_exists($identifier, $voting_log) ? $voting_log[$identifier] : 0;
        // Admin bypass?
        $admin_bypass = current_user_can('administrator') && apply_filters('bbp_voting_admin_bypass', false);
        // View only score?
        $topic_id = bbp_get_reply_topic_id($post_id);
        $topic_status = get_post_status( $topic_id );
        $view_only = ($topic_status == 'closed' && apply_filters('bbp_voting_disable_voting_on_closed_topic', false)) ? true : false;
        // Show labels?
        $show_labels = apply_filters('bbp_voting_show_labels', true);
        echo '<div class="bbp-voting bbp-voting-post-'.$post_id. ($view_only ? ' voted-up voted-down' : ($existing_vote == 1 ? ' voted-up' : ($existing_vote == -1 ? ' voted-down' : ''))) . ($admin_bypass ? ' admin-bypass' : '') .'">';
        //adds the word 'helpful' in red above the arrow
        if($show_labels)
        echo '<div class="bbp-voting-label helpful">'.apply_filters('bbp_voting_helpful', 'Helpful').'</div>';
        if(function_exists( 'is_amp_endpoint' ) && is_amp_endpoint()) {
            // AMP = No JS
            $post_url = admin_url('admin-ajax.php');
            // Up vote
            $plusups = $ups ? '+'.$ups : ' ';
            echo '<form name="amp-form' . $post_id . '" method="post" action-xhr="'.$post_url.'" target="_top" on="submit-success: AMP.setState({\'voteup'. $post_id .'\': '.($ups + 1).'})">
                <input type="hidden" name="action" value="bbpress_post_vote_link_clicked">
                <input type="hidden" name="post_id" value="' . $post_id . '" />
                <input type="hidden" name="direction" value="1" />
                <input type="submit" class="nobutton upvote-amp" value="🔺" />
                <span class="vote up" [text]="voteup'. $post_id .' ? \'+\' + voteup'. $post_id .' : \''.$plusups.'\'">'.$plusups.'</span>
            </form>';
            // Display current vote count for post
            // echo '<div class="score">'. $score. '</div>';
            // echo '<div class="score" style="background-color: rgb('. floor((1 - $score) * 255). ', '.floor($score * 255).', 0); width:'.floor($score * 100).'%;"></div>';
            // Down vote
            echo '<form name="amp-form' . $post_id . '" method="post" action-xhr="'.$post_url.'" target="_top" on="submit-success: AMP.setState({\'votedown'. $post_id .'\': '.($downs - 1).'})">
                <input type="hidden" name="action" value="bbpress_post_vote_link_clicked">
                <input type="hidden" name="post_id" value="' . $post_id . '" />
                <input type="hidden" name="direction" value="-1" />
                <input type="submit" class="nobutton downvote-amp" value="🔻" />
                <span class="vote down" [text]="votedown'. $post_id .' || \''.($downs ? $downs : ' ').'\'">'.($downs ? $downs : '').'</span>
            </form>';
        } else {
            // Normal JS AJAX version
            // Up vote
            echo '<a class="vote up" data-votes="'.($ups ? '+'.$ups : '').'" onclick="bbpress_post_vote_link_clicked(' . $post_id . ', 1); return false;">Up</a>';
            // Display current vote count for post
            echo '<div class="score">'. $score. '</div>';
            // Down vote
            echo '<a class="vote down" data-votes="'.($downs ? $downs : '').'" onclick="bbpress_post_vote_link_clicked(' . $post_id . ', -1); return false;">Down</a>';
        }
        //adds the words 'not helpful' in red below the arrow
        if($show_labels)
        echo '<div class="bbp-voting-label not-helpful">'.apply_filters('bbp_voting_not_helpful', 'Not Helpful').'</div>';
        echo '</div>';
    }
}

// Sort by Votes

add_filter('bbp_has_replies_query', 'sort_bbpress_posts_by_votes', 99);
add_filter('bbp_has_topics_query', 'sort_bbpress_posts_by_votes', 99);

function sort_bbpress_posts_by_votes( $args = array() ) {
    $forum_id = bbp_get_forum_id();
    $this_post_type = bbp_voting_get_current_post_type();
    $forum_post_type = bbp_get_forum_post_type();
    $topic_post_type = bbp_get_topic_post_type();
    switch($this_post_type) {
        case $forum_post_type:
            $post_setting = get_post_meta( $forum_id, 'sort_bbpress_topics_by_votes_on_forum', true);
            $broad_enable = apply_filters('sort_bbpress_topics_by_votes', false);
        break;
        case $topic_post_type:
            $post_setting = get_post_meta( $forum_id, 'sort_bbpress_replies_by_votes_on_forum', true);
            $broad_enable = apply_filters('sort_bbpress_replies_by_votes', false);
        break;
    }
    if(!empty($post_setting)) {
        // Forum-specific override is set (not Default)
        if($post_setting === 'false') return $args;
    } else {
        // Use broad disable settings
        if($broad_enable === false) return $args;
    }
    if(!apply_filters('bbp_voting_allowed_on_forum', true, $forum_id)) return $args;
    // Done with checks... let's do this
    // Find any replies that are missing the bbp_voting_score post meta and fill them with 0
    $args2 = $args;
    $args2['meta_query'] = [
        [
            'key' => 'bbp_voting_score',
            'compare' => 'NOT EXISTS',
            'value' => ''
        ],
    ];
    $query = new WP_Query($args2);
    foreach($query->posts as $reply) {
        update_post_meta($reply->ID, 'bbp_voting_score', '0');
    }

    // Now that all missing scores are filled in, we can sort the original args by the score
    // $args['meta_key'] = 'bbp_voting_score';
    // $args['orderby'] = [
    //     'post_type' => 'DESC', 
    //     'meta_value_num' => 'DESC',
    //     'date' => 'DESC'
    // ];
    unset($args['meta_key']);
    unset($args['meta_type']);
    unset($args['orderby']);
    unset($args['order']);
    $args['meta_query'] = [
        'relation' => 'AND',
        'orderby_score' => [
            'key' => 'bbp_voting_score',
            'type' => 'NUMERIC'
        ]
    ];
    $args['orderby'] = [
        'post_type' => 'DESC', 
        'orderby_score' => 'DESC'
    ];
    if($this_post_type !== $topic_post_type) {
        // Add Freshness as a 3rd orderby for topics on a forum
        $args['meta_query']['orderby_freshness'] = [
            'key' => '_bbp_last_active_time',
            'type' => 'DATETIME',
        ];
        $args['orderby']['orderby_freshness'] = 'DESC';
    }
    return $args;
}

function bbp_voting_get_current_post_type() {
    $this_post_id = bbp_get_reply_id() ?: bbp_get_topic_id();
    return get_post_type($this_post_id);
}